#version 120

/*
DRAWBUFFERS:01

const bool shadowHardwareFiltering0 	= false;
const bool shadowHardwareFiltering1 	= true;

const bool shadowtex0Mipmap 			= true;
const bool shadowtex1Mipmap 			= true;

const bool shadowcolor0Mipmap 			= true;
const bool shadowcolor1Mipmap 			= true;
*/

uniform vec3 upPosition;
uniform vec3 sunPosition;
uniform vec3 cameraPosition;
uniform vec3 shadowLightPosition;

uniform mat4 shadowModelView;
uniform mat4 shadowProjection;
uniform mat4 gbufferModelView;
uniform mat4 gbufferProjection;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjectionInverse;

uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D colortex2;
uniform sampler2D depthtex0;
uniform sampler2D depthtex1;
uniform sampler2D shadowtex0;
uniform sampler2D shadowcolor0;
uniform sampler2D shadowcolor1;

uniform sampler2DShadow shadowtex1;

uniform float frameTime;
uniform float viewWidth;
uniform float viewHeight;
uniform float aspectRatio;
uniform float rainStrength;
uniform float frameTimeCounter;

uniform int isEyeInWater;
uniform int frameCounter;

#define UP_DIRECTION
#define LIGHT_MATRIX
#define SHADOW_MATRIX
#define FOV_PROJECTION

#include "/lib/utilities.glsl"

struct positionStruct {

    vec3 viewSpacePosition;
    vec3 worldSpacePosition;
    vec3 shadowSpacePosition;

} Position;

struct inputStruct {

    mat2x3 colorTexture2;

    vec4 colorTexture0;

    float depthTexture1;

} Input;

#include "/lib/asset/shadow.glsl"
#include "/lib/asset/initialize.glsl"

#include "/lib/utility/misc/ditherIndexMap.glsl"

#include "/lib/utility/sky/computeSky.glsl"
#include "/lib/utility/sky/computeCrepuscularRays.glsl"

void main() {

    float curve = 1.0;

    #ifdef ADAPTIVE_SCALING
        curve = clamp01((FPS_TARGET * 0.75) * frameTime + 0.75) / 0.75;
    #endif

    vec2 compositeCoord = (texCoord / COMPOSITE_RES) * curve;
    
    Input.colorTexture2[0] = textureLoad(colortex2, texCoord).rgb;

    Input.colorTexture0 = textureLoad(colortex0, texCoord);

    Input.colorTexture0.rgb = RGBToYCoCg(Input.colorTexture0.rgb);

    Input.colorTexture0.g = encode2x8(Input.colorTexture0.gb, dither);

    Position.worldSpacePosition = vec3(0.0);
    Position.shadowSpacePosition = vec3(0.0);

    gl_FragData[0] = vec4(0.0);
    gl_FragData[1] = vec4(to16F(Input.colorTexture0.rg), 0.0, Input.colorTexture0.a);

    if (!coordInTile(compositeCoord)) return;

    mat2x4 lightingMatrix = mat2x4(0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0);

    #define AO lightingMatrix[0].w
    #define CR lightingMatrix[1].w

    #define SD lightingMatrix[0].xyz
    #define SB lightingMatrix[1].xyz

    #ifndef SKYLIGHT_DIFFUSE
        SD = computeSky(vec3(0.0)) * 0.5;
    #endif

    Input.colorTexture2[1] = textureLoad(colortex2, compositeCoord).rgb;

    Input.depthTexture1 = textureLoad(depthtex1, compositeCoord).r;

    Position.viewSpacePosition = projectViewSpace(compositeCoord, Input.depthTexture1, FOV, gbufferProjectionInverse);

    ivec2 IDSource = decodeIDs(Input.colorTexture2[1].b + rcp8bit);

    float skyLight = (int(Input.colorTexture2[1].y * 65535) >> 8) * rcp8bit;

    int IDs = IDSource.x;

    bool hand = bool(IDSource.y);
   
    if (!hand) {

        Position.worldSpacePosition = mat3x3(gbufferModelViewInverse) * Position.viewSpacePosition; // Cannot used optimized transMAD function due to 3D rays being offset.
     
        #if CREPUSCULAR_RAYS == 2 || CREPUSCULAR_RAYS == 3
            computeThreeDimensionalRays(CR, compositeCoord, Position);
        #endif

        #if CREPUSCULAR_RAYS == 1 || CREPUSCULAR_RAYS == 3
            computeTwoDimensionalRays(CR, compositeCoord, Position);
        #endif
    }

    #if defined AMBIENT_OCCLUSION || defined SKYLIGHT_DIFFUSE || defined SUNLIGHT_BOUNCE
        if (land) {

            Position.shadowSpacePosition = distortShadowSpace(transMAD(shadowProjMatrix, transMAD(gbufferModelViewInverse, Position.viewSpacePosition)), shadowDistanceSize, inverseDistanceSize) * 0.5 + 0.5;

            vec3 normal = mat3x3(gbufferModelView) * decodeNormal(Input.colorTexture2[1].x);

            const float sampleRCP = RCP(UNIFIED_LIGHTING_SAMPLES);

            float ditherScale = dither * 64;
            float maxShadowLOD = computeMaxLOD(log2(vec2(shadowPixels))) * ln2;

            vec3 tangent = normalize(cross(vec3(0.0, 0.0, 1.0), normal));
            mat3 tbnMatrix = mat3(tangent, cross(normal, tangent), normal);

            for (int i = 0; i < UNIFIED_LIGHTING_SAMPLES; ++i) {

                vec2 offset = hammersleyPointMap(i, sampleRCP * i + 1.0, ditherScale);

                vec3 kernel = tbnMatrix * vec3(offset, fInverseLength(offset));

                vec3 screenPos = kernel * ln2 + (Position.viewSpacePosition * 0.5 + 0.5);

                vec2 offsetCoord = projectViewSpace(screenPos.xy, screenPos.z, FOV, gbufferProjection).xy * 0.5 + 0.5;

                float offsetDepth = textureLoad(depthtex1, offsetCoord).r;

                vec3 offsetViewPosition = projectViewSpace(offsetCoord, offsetDepth, FOV, gbufferProjectionInverse);

                vec3 viewPositionDistance = offsetViewPosition - Position.viewSpacePosition;

                vec3 offsetShadowPosition = distortShadowSpace(transMAD(shadowProjMatrix, transMAD(gbufferModelViewInverse, screenPos * 2.0 - 1.0)), shadowDistanceSize, inverseDistanceSize) * 0.5 + 0.5;

                #ifdef SKYLIGHT_DIFFUSE
                    SD = computeSky(kernel) * sampleRCP + SD;
                #endif

                #ifdef SUNLIGHT_BOUNCE
                    vec3 shadowNormal = mat3x3(shadowModelView) * (textureLoadLOD(shadowcolor1, offsetShadowPosition.xy, maxShadowLOD).rgb * 2.0 - 1.0);

                    if (abs(textureLoadLOD(shadowtex0, offsetShadowPosition.xy, maxShadowLOD).r - offsetShadowPosition.z) * 256 < abs(dot(-shadowNormal.xy, offset) * offsetShadowPosition.z) * shadowNormal.z)
                    SB = textureLoadLOD(shadowcolor0, offsetShadowPosition.xy, maxShadowLOD).rgb * sampleRCP + SB;
                #endif

                #ifdef AMBIENT_OCCLUSION
                    AO = (abs(Input.depthTexture1 - offsetDepth) * sqrt(abs(viewPositionDistance.z)) + abs(dot(normal, viewPositionDistance) * fInverseLength(viewPositionDistance) - 1.0)) * sampleRCP + AO;
                #endif

            }

           AO = pow2(AO * inversesqrt(exp2(AO) * ln2 + rcpln2));
           SB = SB * fLength(SD) * 0.5;
           SD = SD * pow2(skyLight) * 0.5;
        }
    #endif

    gl_FragData[0].rgb = SD * AO + SB;
    gl_FragData[1].b = to16F(CR);
}