#version 120

/*
DRAWBUFFERS:01

const bool colortex1MipmapEnabled = true;
const bool colortex2MipmapEnabled = true;
*/

uniform sampler2D colortex1;
uniform sampler2D colortex2;

uniform float frameTime;
uniform float viewWidth;
uniform float viewHeight;
uniform float aspectRatio;
uniform float rainStrength;
uniform float frameTimeCounter;

uniform int frameCounter;

uniform ivec2 eyeBrightness;

#define LENS_SETTINGS
#define BLOOM_TILE_COMPUTE

#include "/lib/utilities.glsl"

struct inputStruct {

    vec3 colorTexture1;

} Input;

#include "/lib/utility/misc/ditherIndexMap.glsl"

#include "/lib/utility/post/computeLensSimulations.glsl"
#include "/lib/utility/post/computeBloomTile.glsl"

#include "/lib/utility/refract/computeRefractionComposite3.glsl"

void main() {

    float LOD = 0.0;

    mat3x2 newCoord = mat3x2(texCoord, texCoord, texCoord);
    
    #ifdef REFRACTION
        computeRefraction(newCoord, LOD);
    #endif

    vec2 coord = newCoord[1];

    Input.colorTexture1.r = textureLoadLOD(colortex1, newCoord[0], LOD).r;
    Input.colorTexture1.g = textureLoadLOD(colortex1, newCoord[1], LOD).g;
    Input.colorTexture1.b = textureLoadLOD(colortex1, newCoord[2], LOD).b;

    vec3 output0 = vec3(0.0);

    vec2 tileDistance = screenPixelsSize * 256.0 + 2.0;

    #ifdef BLOOM
    	computeBloomTile(output0, 0);
		computeBloomTile(output0, 1);
		computeBloomTile(output0, 2);
		computeBloomTile(output0, 3);
		computeBloomTile(output0, 4);
        
		if (BLOOM_TILES >= 6) computeBloomTile(output0, 5);
		if (BLOOM_TILES >= 7) computeBloomTile(output0, 6);
    #endif

    #ifdef LENS_GLARE
        coord = downScale(texCoord, vec2(2.0, 2.0), vec2(1.0, 1.0));

        if (coordInTile(coord)) {

            const vec2 coordOffset = LENS_GLARE_TYPE > 0 ? vec2(1.0, 0.0) : vec2(0.0, 1.0);
            
            vec2 indexSize = dither * 64 * screenPixelsSize;

            output0 = linear(textureLoad(colortex1,  coordOffset * indexSize + coord).rgb) * 0.5 + output0;
            output0 = linear(textureLoad(colortex1, -coordOffset * indexSize + coord).rgb) * 0.5 + output0;

        }
    #endif

    #if LENS_FLARE_TYPE > 1 || defined LENS_DIFFRACTION
        coord = downScale(texCoord, tileDistance, vec2(tileDistance.x - 1.0, 0.0));

        if (coordInTile(coord)) {
            
            #if LENS_FLARE_TYPE > 1
                float curve = pow2(1.0 - fLength(coord * 2.0 - 1.0));
                float inverseCurve = (1.0 - curve);

                if (curve > 0.0625) {
    
                    vec2 offset = coord - 0.5;

                    mat3x4 curved = mat3x4(0.0);
                    mat3x4 curvedInv = mat3x4(0.0);

                    curved[0] = computeFlareProperties(59.82 * curve * 2.7, coord, offset);
                    curved[1] = computeFlareProperties(45.33 * curve * 1.2, coord, offset);
                    curved[2] = computeFlareProperties(28.62 * curve * 1.4, coord, offset);

                    curvedInv[0] = computeFlareProperties(24.16 * inverseCurve * 2.3, coord, offset);
                    curvedInv[1] = computeFlareProperties(17.43 / inverseCurve * 1.8, coord, offset);
                    curvedInv[2] = computeFlareProperties(14.01 * inverseCurve * 1.3, coord, offset);

                    int colorCorrect = 0;
                    
                    for (int i = 1; i <= RINGS; ++i) 

                    for (int j = 0; j < i * LENS_EFFECT_SAMPLES * 0.25; ++j) {

                        bokehPoints = bokehPointMap(dither * sampleLOD + j, i, piScale, tauScale) * bokehSize;

                        if (curved[0].w > 0) output0 = computeFlareSample(curved[0].z, curved[0].xy, bokehPoints) * curve + output0;
                        if (curved[1].w > 0) output0 = computeFlareSample(curved[1].z, curved[1].xy, bokehPoints) * curve + output0;
                        if (curved[2].w > 0) output0 = computeFlareSample(curved[2].z, curved[2].xy, bokehPoints) * curve + output0;

                        if (curvedInv[0].w > 0) output0 = computeFlareSample(curvedInv[0].z, curvedInv[0].xy, bokehPoints) * curve + output0;
                        if (curvedInv[1].w > 0) output0 = computeFlareSample(curvedInv[1].z, curvedInv[1].xy, bokehPoints) * curve + output0;
                        if (curvedInv[2].w > 0) output0 = computeFlareSample(curvedInv[2].z, curvedInv[2].xy, bokehPoints) * curve + output0;
                                                
                        ++colorCorrect;
                    }

                    output0 /= colorCorrect * 8.0;

                    output0 *= sqrt(curve + inverseCurve);

                }
            #endif
            
            #ifdef LENS_DIFFRACTION
                computeDiffraction(output0, coord);
            #endif

        }
    #endif

    #ifdef LENS_DIRT
        coord = downScale(texCoord, vec2(tileDistance.x, 2.0), vec2(0.0, 1.0));

        if (coordInTile(coord)) computeDirtyLens(output0, coord);
    #endif

    output0 = sqrt(abs(output0));
    
    float luminance = luma(output0);

    output0 = output0 * (luminance * fExp(-(fPow(luminance, overflow) * overflow + overflowBase)) + lift);

    gl_FragData[0] = vec4(Input.colorTexture1, textureLoad(colortex2, newCoord[1]).g);
    gl_FragData[1] = vec4(output0, 1.0);
}