#version 120

/*
DRAWBUFFERS:03

const bool colortex0MipmapEnabled = true;
const bool colortex1MipmapEnabled = true;
const bool colortex2MipmapEnabled = true;
const bool colortex3MipmapEnabled = true;
*/

uniform sampler2D colortex0;
uniform sampler2D colortex1;
uniform sampler2D colortex3;
uniform sampler2D depthtex1;

uniform float frameTime;
uniform float viewWidth;
uniform float viewHeight;
uniform float smoothMask;
uniform float aspectRatio;
uniform float frameTimeCounter;

uniform int frameCounter;
uniform int isEyeInWater;

#define LENS_SETTINGS
#define BLOOM_TILE_LOAD

#include "/lib/utilities.glsl"

struct inputStruct {

    vec3 colorTexture0;

	float depthTexture1;

} Input;

#include "/lib/utility/misc/ditherIndexMap.glsl"
#include "/lib/utility/misc/computeRandomCoord.glsl"

#include "/lib/utility/post/fetchBloomTile.glsl"
#include "/lib/utility/post/computeDepthOfField.glsl"

void main() {

    Input.colorTexture0 = textureLoad(colortex0, texCoord).rgb;

	Input.depthTexture1 = textureLoad(depthtex1, texCoord).r;

	bool hand = textureLoad(colortex0, texCoord).a > 0;

	vec2 tileDistance = screenPixelsSize * 256.0 + 2.0;

	mat4x2 coords = mat4x2(0.0);

	coords[0] = upScale(texCoord, vec2(2.0), vec2(0.5));
	coords[1] = upScale(texCoord, vec2(2.0), vec2(1.0));

	coords[2] = upScale(texCoord, vec2(tileDistance.x, 2.0), vec2(0.0, 1.0));
	
	coords[3] = upScale(texCoord, tileDistance, vec2(tileDistance.x - 1.0, 0.0));

    vec3 tiledEffects = vec3(0.0);
	vec3 lensDirt = vec3(0.0);

	float maxLOD = computeMaxLOD(log2(screenPixels));

	float mixTime = (1.0 - frameTime);

	float smoothBuffer = luma(textureLoadLOD(colortex0, computeRandomCoord(texCoord), maxLOD).rgb);
	
	bool depthPixel = all(lessThanEqual(abs(texCoord - screenPixelsSize), screenPixelsSize));

	if (depthPixel) {

		smoothBuffer = textureLoad(depthtex1, vec2(0.5)).r;

		mixTime = sqrt(exp2(-mixTime) * phi - frameTime);

	}

	smoothBuffer = mix(smoothBuffer, textureLoad(colortex3, texCoord).a, mixTime);

	#ifdef ADVANCED_FILTERING
        float imbalance = abs(dither - 0.5) * 2.0 + 1.0;

		float variableLOD = (imbalance * 0.5 + 0.5) / 0.75;

        vec2 ditherOffset = imbalance * vec2(1.0, aspectRatio);
	#else
	 	const float staticLOD = 2.0 / 0.75;
	#endif

	#ifdef BLOOM
		#ifdef LENS_DIRT
			#ifdef ADVANCED_FILTERING
				lensDirt = textureLoadLOD(colortex1, vec2( ditherOffset.x,  ditherOffset.y) * screenPixelsSize + coords[2], variableLOD).rgb * 0.25 + lensDirt;
				lensDirt = textureLoadLOD(colortex1, vec2(-ditherOffset.x,  ditherOffset.y) * screenPixelsSize + coords[2], variableLOD).rgb * 0.25 + lensDirt;
				lensDirt = textureLoadLOD(colortex1, vec2( ditherOffset.x, -ditherOffset.y) * screenPixelsSize + coords[2], variableLOD).rgb * 0.25 + lensDirt;
				lensDirt = textureLoadLOD(colortex1, vec2(-ditherOffset.x, -ditherOffset.y) * screenPixelsSize + coords[2], variableLOD).rgb * 0.25 + lensDirt;
			#else
				lensDirt = textureLoadLOD(colortex1, coords[2], staticLOD).rgb;
			#endif
		#else
			lensDirt = vec3(0.0);
		#endif
		
		lensDirt = linear(lensDirt);

		fetchBloomTile(tiledEffects, 0);
		fetchBloomTile(tiledEffects, 1);
		fetchBloomTile(tiledEffects, 2);
		fetchBloomTile(tiledEffects, 3);
		fetchBloomTile(tiledEffects, 4);
		
		if (BLOOM_TILES >= 6) fetchBloomTile(tiledEffects, 5);
		if (BLOOM_TILES >= 7) fetchBloomTile(tiledEffects, 6);

		tiledEffects *= exp2(-BLOOM_TILES) * BLOOM_TILES + lensDirt;

		#if LENS_FLARE_TYPE > 0 && defined LENS_DIRT
			vec3 chromaHoop = vec3(0.0);

			mat3x2 chromaCoords = mat3x2(0.0);

			chromaCoords[0] = -texCoord + 1.0;

			chromaCoords[1] = fNormalize(0.5 - chromaCoords[0]);

			chromaCoords[2] = fract(chromaCoords[1] * 0.5 + chromaCoords[0]);

			vec2 disperse = screenPixelsSize * 32.0;

			chromaHoop.r = textureLoadLOD(colortex1, upScale(chromaCoords[1] * -disperse + chromaCoords[2], 4, bloomTilePositions[0]), 4).r;
			chromaHoop.g = textureLoadLOD(colortex1, upScale(chromaCoords[2] 							  , 4, bloomTilePositions[0]), 4).g;
			chromaHoop.b = textureLoadLOD(colortex1, upScale(chromaCoords[1] *  disperse + chromaCoords[2], 4, bloomTilePositions[0]), 4).b;

			chromaHoop = linear(chromaHoop * fLength(texCoord - 0.5) * lensDirt * 255);

			tiledEffects = chromaHoop * generate2DNoise(chromaCoords[1] * 512.0 + vec2(-time, time)) + tiledEffects;
		#endif
	#endif
	
	#ifdef LENS_DIRT
		#if LENS_FLARE_TYPE > 1 || defined LENS_DIFFRACTION
			vec3 advancedFlare = vec3(0.0);

			#ifdef ADVANCED_FILTERING
				advancedFlare = textureLoadLOD(colortex1, vec2( ditherOffset.x,  ditherOffset.y) * screenPixelsSize + coords[3], variableLOD).rgb * 0.25 + advancedFlare;
				advancedFlare = textureLoadLOD(colortex1, vec2(-ditherOffset.x,  ditherOffset.y) * screenPixelsSize + coords[3], variableLOD).rgb * 0.25 + advancedFlare;
				advancedFlare = textureLoadLOD(colortex1, vec2( ditherOffset.x, -ditherOffset.y) * screenPixelsSize + coords[3], variableLOD).rgb * 0.25 + advancedFlare;
				advancedFlare = textureLoadLOD(colortex1, vec2(-ditherOffset.x, -ditherOffset.y) * screenPixelsSize + coords[3], variableLOD).rgb * 0.25 + advancedFlare;
			#else
				advancedFlare = textureLoadLOD(colortex1, coords[3], staticLOD).rgb;
			#endif

			tiledEffects = linear(advancedFlare) * mix(1.0, generate2DNoise(fNormalize(0.5 - texCoord) * screenPixels), fLength(texCoord * 2.0 - 1.0)) + tiledEffects;
		#endif
	#endif

	#ifdef LENS_GLARE
		const float sampleCount = LENS_EFFECT_SAMPLES * 0.125;

		tiledEffects *= sampleCount * 2.0;

		const float sampleRCP = RCP(sampleCount);

		vec2 coordOffset = vec2(0.0);
		
		vec2 blurSize = screenPixelsSize * sampleRCP * dither * 8;

		for (int i = 1; i <= sampleCount; ++i) {

			coordOffset = vec2(0.0, i);

			if (LENS_GLARE_TYPE > 0)
			coordOffset = coordOffset.yx;
			
			#ifdef ADVANCED_FILTERING
				tiledEffects = linear(textureLoadLOD(colortex1,  coordOffset * blurSize + coords[1], variableLOD).rgb) + tiledEffects;
				tiledEffects = linear(textureLoadLOD(colortex1, -coordOffset * blurSize + coords[1], variableLOD).rgb) + tiledEffects;
			#else
				tiledEffects = linear(textureLoadLOD(colortex1,  coordOffset * blurSize + coords[1], staticLOD).rgb) + tiledEffects;
				tiledEffects = linear(textureLoadLOD(colortex1, -coordOffset * blurSize + coords[1], staticLOD).rgb) + tiledEffects;
			#endif

		}

		tiledEffects *= sampleRCP * 0.5;
	#endif

	#ifdef DEPTH_OF_FIELD
		computeDepthOfField(Input.colorTexture0, Input.depthTexture1, hand);
	#endif

	#ifdef AUTO_EXPOSURE
		#ifndef CAMERA_ARTIFACT_CORRECTION
			vec2 EV = vec2(textureLoad(colortex3, texCoord).a, 0.0);

			if (depthPixel) EV.x = textureLoad(colortex3, screenPixelsSize).a;
		#else
			vec2 EV = vec2(textureLoadLOD(colortex3, coords[0], maxLOD).a, 0.0);
		#endif
		
		EV.x = RCP(EV.x * 4.0 + rcp16bit);

		EV.y = max(EV.x - inversesqrt(EV.x + 1.0), fLength(tiledEffects + sqrt(sunLuminance * sqrt(moonLuminance))) / sunLuminance);
		
		EV.x /= max(log2(EV.x), 0.0) + 1.0;
	#else
		vec2 EV = vec2(1.0);

		EV.x = RCP((pow2(APERTURE) * log2(float(SHUTTER_SPEED))) / (ISO * 0.5));

		EV.y = max(EV.x - inversesqrt(EV.x + 1.0), fLength(tiledEffects + sqrt(sunLuminance * sqrt(moonLuminance))) / sunLuminance);
	#endif

	EV = sqrt(abs(EV) * 0.5) / 0.8333;

	EV.y *= inversesqrt(pow2(EV.x * 2.0) * 0.8333 + 1.0);

	EV.x *= EV_MULTIPLIER;

	Input.colorTexture0 = (sqrt(abs(tiledEffects)) * EV.y + Input.colorTexture0) * EV.x;
	
    gl_FragData[0] = vec4(Input.colorTexture0, !hand);
	gl_FragData[1] = vec4(textureLoad(colortex3, texCoord).rgb, smoothBuffer);
}