float dotX(in vec2 x) {
    return dotXSource(x);
}

float dotX(in vec3 x) {
    return dotXSource(x);
}

float dotX(in vec4 x) {
    return dotXSource(x);
}

float fExp(in float x) {
    return fExpSource(x);
}

vec2 fExp(in vec2 x) {
    return fExpSource(x);
}

vec3 fExp(in vec3 x) {
    return fExpSource(x);
}

float fLength(in vec4 x) {
    return fLengthSource(x);
}

float fLength(in vec3 x) {
    return fLengthSource(x);
}

float fLength(in vec2 x) {
    return fLengthSource(x);
}

float fInverseLength(in vec4 x) {
    return fInverseLengthSource(x);
}

float fInverseLength(in vec3 x) {
    return fInverseLengthSource(x);
}

float fInverseLength(in vec2 x) {
    return fInverseLengthSource(x);
}

vec2 fNormalize(in vec2 x) {
    return fNormalizeSource(x);
}

vec3 fNormalize(in vec3 x) {
    return fNormalizeSource(x);
}

vec4 fNormalize(in vec4 x) {
    return fNormalizeSource(x);
}

float computeMaxLOD(in vec2 pixelTileSize) {
    return floor(pixelTileSize.y - (pixelTileSize.y / pixelTileSize.x));
}

bool coordInTile(in vec2 x) {
    return floor(x) == vec2(0);
}

bool coordInTile(in vec2 x, in vec2 y) {
    return all(lessThan(abs(x - 0.5), vec2(y + 0.5)));
}

float pow2(in float x) { return x * x; }
float pow3(in float x) { return x * x * x; }
float pow4(in float x) { return x * x * x * x; }
float pow5(in float x) { return x * x * x * x * x; }
float pow6(in float x) { return x * x * x * x * x * x; }
float pow7(in float x) { return x * x * x * x * x * x * x; }
float pow8(in float x) { return x * x * x * x * x * x * x * x; }

vec3 linear(in vec3 x) { return x * x; }

float generate2DNoise(in vec2 coord) {
    
	mat3x2 value = mat3x2(floor(coord), fract(coord), vec2(0.0));

    value[1] = (value[1] * value[1]) * (3.0 - 2.0 * value[1]);

	value[2].x = mix(random(value[0]                 ), random(value[0] + vec2(1.0, 0.0)), value[1].s);
	value[2].y = mix(random(value[0] + vec2(0.0, 1.0)), random(value[0] + vec2(1.0, 1.0)), value[1].s);
	
	return mix(value[2].x, value[2].y, value[1].y);
}

float generate3DNoise(in vec3 coord) {

    mat2x3 coords = mat2x3(floor(coord), fract(coord));

    mat3x2 value = mat3x2(0.0);

    value[0] = ((coords[0].z) * 19.9 + coords[0].st) + coords[1].st;
    value[1] = ((coords[0].z + 1.0) * 19.9 + coords[1].st) + coords[0].st;
    
    value[2].x = generate2DNoise(value[0]);
    value[2].y = generate2DNoise(value[1]);

    return mix(value[2].x, value[2].y, coords[1].z);
}

vec3 textureLoadInterlaced(in sampler2D sampleBuffer, in vec2 coord, in vec2 screenPixelsSize, in ivec2 table, in vec2 initialSample) {

    vec3 unlacedBuffer = vec3(0.0);
    
    mat4x2 lumaOffset = mat4x2(0.0);

    lumaOffset[0] = textureLoad(sampleBuffer, ivec2( 1,  0) * screenPixelsSize + coord).rg;
    lumaOffset[1] = textureLoad(sampleBuffer, ivec2(-1,  0) * screenPixelsSize + coord).rg;
    lumaOffset[2] = textureLoad(sampleBuffer, ivec2( 0,  1) * screenPixelsSize + coord).rg;
    lumaOffset[3] = textureLoad(sampleBuffer, ivec2( 0, -1) * screenPixelsSize + coord).rg;
    
    vec4 redLace = 1.0 - abs(vec4(lumaOffset[0].r, lumaOffset[1].r, lumaOffset[2].r, lumaOffset[3].r) - initialSample.r);

    unlacedBuffer.b = dot(redLace / dotX(redLace), vec4(lumaOffset[0].g, lumaOffset[1].g, lumaOffset[2].g, lumaOffset[3].g));

    unlacedBuffer.g = initialSample.g;

    return vec3(initialSample.r, table.x == table.y ? unlacedBuffer.bg : unlacedBuffer.gb);
}

vec3 distortShadowSpace(in vec3 worldSpaceInput, in float distortionCoeff, in float invDistortionCoeff) {

	worldSpaceInput.xy /= fLength(worldSpaceInput.xy) * distortionCoeff + invDistortionCoeff;

    worldSpaceInput.z  *= zScaleCoeff;

	return worldSpaceInput;
}

vec3 projectViewSpace(in vec2 coord, in float depth, in float FOV, in mat4 projectionMatrix) {

    vec3 screenPos = vec3(coord, depth) * 2.0 - 1.0;

    vec3 viewSpacePosition = projMAD3(projectionMatrix, screenPos) / (projectionMatrix[2].w * screenPos.z + projectionMatrix[3].w);
    
    viewSpacePosition.xy *= FOV;

	return viewSpacePosition;
}

/*
Encode formula

bitfield = round a * 2^halfbits-1

X : bitfield | Y
Y : bitfield << halfbits

XY : XY / 2^bits-1
*/

/*
Decode formula

bitfield : a * 2^bits-1

X : bitfield & 2^halfbits-1
Y : bitfield >> halfbits

XY : XY / 2^halfbits-1
*/

float encode2x8(in vec2 a, in float mantissa) {

    a = abs((mantissa - 0.5) * rcp8bit + a);
    
    ivec2 bitfield = ivec2(round(a * 255));

    return float(bitfield.x | bitfield.y << 8) * rcp16bit;
}

float encodeNormal(in vec3 normal) {

    normal.xy = normal.xy / dot(abs(normal), vec3(1.0));

    normal.xy = normal.z <= 0.0 ? (1.0 - abs(normal.yx)) * fSign(normal.xy) : normal.xy;

    return encode2x8(normal.xy * 0.5 + 0.5, 0.5);
}

vec2 decode2x8(in float a) {

    int bitfield = int(a * 65535);

    return vec2(bitfield & 255, bitfield >> 8) * rcp8bit;
}

ivec2 decodeIDs(in float packedIDs) {

    float bitfield = round(packedIDs * 8);
    
    return ivec2(bitfield, abs(packedIDs * 8 - bitfield) >= 0.5 - rcp8bit * 0.125);
}

vec3 decodeNormal(in float encodedNormal) {

    vec3 normal = vec3(0.0);

    vec2 bitDecoded = decode2x8(encodedNormal) * 2.0 - 1.0;

    normal.xy = abs(bitDecoded);

    normal.z = 1.0 - normal.x - normal.y;
    
    normal.xy = (1.0 - normal.yx) * fSign(bitDecoded);

    normal.xy = normal.z < 0.0 ? normal.xy : bitDecoded;

    return fNormalize(normal.xyz);
}

int bitfieldReverse(in int point) {

    point = ((point & 0x55555555) << 1 ) | ((point & 0xAAAAAAAA) >> 1 );
    point = ((point & 0x33333333) << 2 ) | ((point & 0xCCCCCCCC) >> 2 );
    point = ((point & 0x0F0F0F0F) << 4 ) | ((point & 0xF0F0F0F0) >> 4 );
    point = ((point & 0x00FF00FF) << 8 ) | ((point & 0xFF00FF00) >> 8 );
    point = ((point & 0x0000FFFF) << 16) | ((point & 0xFFFF0000) >> 16);

    return point;
}

vec2 rotateVector(in vec2 x, in float y) {
    return (x - 0.5) * mat2(cos(y), sin(y), -sin(y), cos(y)) + 0.5;
}

vec2 circleMap(in float point) {
    return vec2(cos(point), sin(point));
}

vec2 rotatePoint(in float point) {
    return vec2(cos(point), sin(point)) * point;
}

vec2 hammersleyPointMap(in int point, in float progress, in float ditherScale) {
    return circleMap(bitfieldReverse(point) * bitfieldTau + ditherScale) * progress;
}

vec2 latticePointMap(in float point, in float totalPointsSqrt, in float totalPointsInvSqrt, in float totalPointsRCP) {
    return vec2(mod(point * pi, totalPointsSqrt) * totalPointsInvSqrt, point * totalPointsRCP);
}

vec2 temporalJitter(in int frameCounter, in float frameTime) {
    return circleMap((frameCounter & int(0.0625 / frameTime - 1)) + 1);
    // Cycles per frame must be atleast 1/16th of the frame-rate for a stable image with very low flickering.
}

vec3 RGBToYCoCg(in vec3 RGB) {

	RGB = vec3(
    0.25 * RGB.r + 0.5 * RGB.g + 0.25 * RGB.b, 
    0.5 * RGB.r - 0.5 * RGB.b,
    -0.25 * RGB.r + 0.5 * RGB.g - 0.25 * RGB.b
    );
    
    RGB = RGB + vec3(0.0, 0.5, 0.5);

    return RGB;
}

vec3 YCoCgToRGB(in vec3 YCoCg) {

    YCoCg = YCoCg - vec3(0.0, 0.5, 0.5);

    YCoCg = vec3(
    YCoCg.r + YCoCg.g - YCoCg.b,
    YCoCg.r + YCoCg.b,
    YCoCg.r - YCoCg.g - YCoCg.b
    );

    return YCoCg;
}

vec3 blackbody(const in float t) {

    const vec4 vx = vec4(-0.2661239e9, -0.2343580e6, 0.8776956e3, 0.179910);
    const vec4 vy = vec4(-1.1063814, -1.34811020, 2.18555832, -0.20219683);

    const float it = 1.0 / t;
    const float it2 = it * it;

    const float x = dot(vx, vec4(it*it2, it2, it, 1.0));
    const float x2 = x * x;
    
    const float y = dot(vy, vec4(x*x2, x2, x, 1.0));
    
    return vec3(y / x, 1.0, y / (1.0 - x - y));
}

bool emissiveCheck(in float IDs) {
    return emissive;
}

bool waterCheck(in float IDs) {
    return water;
}