#ifdef PIXEL
    uniform sampler2D texture;
    uniform sampler2D normals;

    varying vec4 biomeColor;
    varying vec3 vertexPos;
    varying vec2 lightmap;

    flat varying vec4 vertexData;

    uniform mat4 gbufferModelViewInverse;

    #ifdef ENTITIES
        uniform vec4 entityColor;
    #endif

    #if defined HAND || defined HAND_WATER
        uniform int heldItemId;
    #endif

    #ifdef WATER 
        uniform int isEyeInWater;
    #endif

    #include "/lib/utilities.glsl"
    
    #if defined TERRAIN || defined HAND
        varying mat3x2 parallaxData;

        flat varying mat3x3 tbn;

        #include "/lib/utility/misc/terrainParallax.glsl"
    #endif

    void main() {

        vec4 color = texture2D(texture, texCoord) * biomeColor;
        
        vec3 normal = vertexData.xyz;

        float layer = 1.0;
        float hand = 0.0;

        float IDs = round(vertexData.w);

        #ifdef ENTITIES
            color.rgb = mix(color.rgb, entityColor.rgb, entityColor.a);
        #endif

        #ifdef GLINT
            normal = vec3(0.0);
        #endif

        #if defined HAND || defined HAND_WATER
            if (heldItemId < 256) 
            IDs = heldItemId;
            
            hand = 1.0;
        #endif

        #ifdef WATER
            if (waterSource) color = vec4(exp2(-waterColor.rgb), waterColor.a);

            if (underWater || IDs == 79.0 || IDs == 90.0 || IDs == 160.0 || IDs == 174.0 || glassSource) 
            layer = 0.0;
        #endif

        #if defined WATER || defined HAND_WATER
            if (glassSource) color.a = floor(color.a + 0.5) - 0.5;
        #endif

        #ifdef WEATHER
            color = vec4(exp2(-waterColor.rgb), waterColor.a) * color.a;
            
            color.a = 0.5 * color.a;

            layer = 0.0;
        #endif

        #ifdef BEACONBEAM
            IDs = 138;
            layer = 0.0;
        #endif

        IDs = max(IDs, 1);

        if (IDs == 2) IDs = 1;
        if (plantSource) IDs = 2;

        if (IDs == 3) IDs = 1;
        if (emissiveSource && !warmSource) IDs = 3;

        if (IDs == 4) IDs = 1;
        if (warmSource) IDs = 4;

        if (IDs == 5) IDs = 1;
        if (waterSource) IDs = 5;

        if (IDs == 6) IDs = 1;
        if (glassSource) IDs = 6;

        if (IDs > 6) IDs = 1;

        IDs = (hand * 0.5 + IDs) / 8;
        
        #if defined TERRAIN || defined HAND
            vec2 parallaxCoord = texCoord;

            mat2x2 wrapTexture = mat2x2(0.0);

            wrapTexture[0] = dFdx(texCoord);
            wrapTexture[1] = dFdy(texCoord);

            vec3 vertexDist = fNormalize(vertexPos);

            #ifdef UNIVERSAL_PARALLAX
                parallaxCoord = computeParallaxCoord(wrapTexture, tbn * vertexDist);

                color = texture2DGradARB(texture, parallaxCoord, wrapTexture[0], wrapTexture[1]) * biomeColor;
            #endif
            
            #ifdef UNIVERSAL_BUMP
                #if BUMP_MODE == 0
                    normal.y = fLength(texture2DGradARB(texture, parallaxCoord, wrapTexture[0], wrapTexture[1]).rgb);
                    
                    normal.x = fLength(texture2DGradARB(texture, vec2(vertexDist.x * 8.0, 0.0) * rcp16bit + parallaxCoord, wrapTexture[0], wrapTexture[1]).rgb);
                    normal.z = fLength(texture2DGradARB(texture, vec2(0.0, vertexDist.z * 8.0) * rcp16bit + parallaxCoord, wrapTexture[0], wrapTexture[1]).rgb);

                    normal.xy = (normal.y - normal.xz) / (vertexDist.xz * rcp16bit * 64.0);
                #else
                    normal.xy = texture2DGradARB(normals, parallaxCoord, wrapTexture[0], wrapTexture[1]).rg * 2.0 - 1.0;
                #endif

                normal.z = sqrt(abs(1.0 - dotX(normal.xy)));

                normal = normal * tbn; 
            #endif
        #endif

        normal = mat3(gbufferModelViewInverse) * normal;
        
        gl_FragData[0] = color;
        gl_FragData[1] = vec4(encodeNormal(normal), encode2x8(lightmap, 0.5), IDs, layer);
    }
#endif

#ifdef VERTEX
attribute vec4 mc_Entity;
attribute vec4 at_tangent;
attribute vec4 mc_midTexCoord;

    varying vec4 biomeColor;
    varying vec3 vertexPos;
    varying vec2 lightmap;

    flat varying vec4 vertexData;

    #if defined TERRAIN || defined HAND || defined HAND_WATER
        uniform vec3 cameraPosition;

        uniform mat4 gbufferModelViewInverse;
        uniform mat4 gbufferModelView;

        uniform float frameTimeCounter;
    #endif

    #include "/lib/utilities.glsl"

    #if defined TERRAIN
        #include "/lib/utility/misc/displaceVertex.glsl"
    #endif
    
    #if ANTI_ALIASING == 2 || ANTI_ALIASING == 3
        uniform int frameCounter;

        uniform float viewWidth;
        uniform float viewHeight;
        uniform float frameTime;
    #endif
    
    #if defined TERRAIN || defined HAND
        varying mat3x2 parallaxData;

        flat varying mat3x3 tbn;
    #endif

    void main() {

        texCoord        = gl_MultiTexCoord0.xy;
        lightmap        = gl_MultiTexCoord1.xy * rcp8bit + rcp8bit;
        biomeColor      = gl_Color;

        vertexData.w    = mc_Entity.x;
        vertexData.xyz  = gl_NormalMatrix * gl_Normal;

        gl_Position = transMAD(gl_ModelViewMatrix, gl_Vertex.xyz).xyzz;

        #if defined TERRAIN && defined VERTEX_DISPLACEMENT
            gl_Position = transMAD(gbufferModelViewInverse, gl_Position.xyz).xyzz;

            if (
            mc_Entity.x == 141.0 ||
            mc_Entity.x == 142.0 ||
            mc_Entity.x == 115.0 ||
            mc_Entity.x == 105.0 ||
            mc_Entity.x == 104.0 ||
            mc_Entity.x == 59.0  ||
            mc_Entity.x == 51.0  ||
            mc_Entity.x == 40.0  ||
            mc_Entity.x == 39.0  ||
            mc_Entity.x == 38.0  ||
            mc_Entity.x == 37.0  ||
            mc_Entity.x == 31.0  ||
            mc_Entity.x == 32.0  ||
            mc_Entity.x == 30.0  ||
            mc_Entity.x == 6.0	 )
            displaceVertex(gl_Position.xyz, lightmap.y * 0.75, true);

            if (
            mc_Entity.x == 175.0 ||
            mc_Entity.x == 161.0 || 
            mc_Entity.x == 106.0 ||
            mc_Entity.x == 83.0  ||
            mc_Entity.x == 18.0  )
            displaceVertex(gl_Position.xyz, lightmap.y * 0.25, false);

            if (
            mc_Entity.x == 141.0 ||
            mc_Entity.x == 142.0 ||
            mc_Entity.x == 115.0 ||
            mc_Entity.x == 59.0  ||
            mc_Entity.x == 38.0  ||
            mc_Entity.x == 37.0  ||
            mc_Entity.x == 31.0  &&
            texCoord.y < mc_midTexCoord.y) {

                float positionLength = fLength(gl_Position.xyz);

                if (positionLength < 1.0) gl_Position.xz *= 2.0 - min(positionLength, 1.0);

            }

            gl_Position = transMAD(gbufferModelView, gl_Position.xyz).xyzz;
        #endif 

        #if defined HAND && defined VERTEX_DISPLACEMENT
            gl_Position = transMAD(gbufferModelViewInverse, gl_Position.xyz).xyzz;

            gl_Position.xyz = vec2(sin(time), cos(time)).xyx * vec3(0.0025, 0.0075, 0.0000625) + gl_Position.xyz;

            gl_Position = transMAD(gbufferModelView, gl_Position.xyz).xyzz;
        #endif    
        
        #if defined TERRAIN || defined HAND
            parallaxData[1] = mc_midTexCoord.xy;

            vec2 midCoord = texCoord - parallaxData[1];

            parallaxData[2] = fSign(midCoord) * 0.5 + 0.5;
            parallaxData[0] = abs(midCoord) * 2.0;
            parallaxData[1] = min(texCoord, parallaxData[1] - midCoord);

            vec3 tangent = fNormalize(gl_NormalMatrix * at_tangent.xyz);

            tbn = transpose(mat3x3(tangent, cross(tangent, vertexData.xyz), vertexData.xyz));
                
            vertexPos = gl_Position.xyz;
        #endif 

        gl_Position = gl_Position.xyzz * diagonal4(gl_ProjectionMatrix) + vec4(0, 0, gl_ProjectionMatrix[3].z, 0);

        #if ANTI_ALIASING == 2 || ANTI_ALIASING == 3
            gl_Position.xy = (temporalJitter(frameCounter, frameTime) / vec2(viewWidth, viewHeight)) * gl_Position.w + gl_Position.xy;
        #endif
        
    }
#endif