varying vec2 texCoord;

#include "/lib/asset/settings.glsl"

#include "/lib/asset/values.glsl"

#include "/lib/asset/macros.glsl"

#include "/lib/asset/helpers.glsl"

#if !defined VERTEX && !defined PIXEL
    vec2 screenPixels = vec2(viewWidth, viewHeight);

    vec2 screenPixelsSize = RCP(screenPixels);
#endif

#ifdef UP_DIRECTION
    vec3 upDirection = fNormalize(upPosition);
#endif

#ifdef LIGHT_MATRIX
    mat2x3 lightPosition = mat2x3(sunPosition * 0.01, shadowLightPosition * 0.01);
#endif

#ifdef SHADOW_MATRIX
    mat4 shadowProjMatrix = shadowProjection * shadowModelView;
#endif

#ifdef FOV_PROJECTION
    float FOV = underWater ? gbufferProjection[1][1] * tan(atan(gbufferProjectionInverse[1][1]) * 0.85) : 1.0;
#endif

#if defined BLOOM_TILE_LOAD || defined BLOOM_TILE_COMPUTE
    const ivec2 bloomTilePositions[7] = ivec2[7](
    ivec2(0, 0),
    ivec2(2, 0),
    ivec2(6, 0),
    ivec2(0, 8),
    ivec2(2, 16),
    ivec2(6, 32),
    ivec2(14, 64));
#endif

#ifdef LENS_SETTINGS
    uniform mat4 gbufferProjectionInverse;

    const vec3 internalScale = vec3((pi / BLADES) * (1.0 - ROUNDING), cos(pi / BLADES), pi * BLADES);

    const int sampleLOD = 32 / LENS_EFFECT_SAMPLES;
    
    float viewDiag = fLength(screenPixels);

	float focalLengthMM = (viewDiag / (tau * atan(gbufferProjectionInverse[1][1]) / 180.0)) / (viewDiag * viewWidth / aspectRatio);
    
    float fStop = focalLengthMM / APERTURE;
    
    #ifdef REFRACTION_CHROMATIC_DISPERSION
        float disperse = exp2(-sqrt(fStop)) * sqrt(fStop) * APERTURE * ln2;

        vec3 dispersion = 1.0 - vec3(disperse, 0.0, -disperse);
    #else
        const vec3 dispersion = vec3(1.0, 1.0, 1.0);
    #endif

    vec2 bokehPointMap(in float point, in float progress, const in float piScale, const in float tauScale) {
        return circleMap(point * tauScale + ROTATION) * abs(internalScale.y * tau - fPow(abs(cos(point * piScale)), internalScale.x)) * progress / (RINGS * pi);
    }
#endif