/* Based off : NVIDIA FXAA v3.11  */

void FXAACompute(inout vec3 output0, inout vec2 texCoord) {

    float lumaM    = luma(output0);

    float lumaS    = luma(textureLoad(colortex0, ivec2( 0, 1) * screenPixelsSize + texCoord).rgb);
    float lumaE    = luma(textureLoad(colortex0, ivec2( 1, 0) * screenPixelsSize + texCoord).rgb);
    float lumaN    = luma(textureLoad(colortex0, ivec2( 0,-1) * screenPixelsSize + texCoord).rgb);
    float lumaW    = luma(textureLoad(colortex0, ivec2(-1, 0) * screenPixelsSize + texCoord).rgb);

    float lumaNW   = luma(textureLoad(colortex0, ivec2(-1,-1) * screenPixelsSize + texCoord).rgb);
    float lumaSE   = luma(textureLoad(colortex0, ivec2( 1, 1) * screenPixelsSize + texCoord).rgb);
    float lumaNE   = luma(textureLoad(colortex0, ivec2( 1,-1) * screenPixelsSize + texCoord).rgb);
    float lumaSW   = luma(textureLoad(colortex0, ivec2(-1, 1) * screenPixelsSize + texCoord).rgb);

    float lumaNS   = lumaN  + lumaS;
    float lumaWE   = lumaW  + lumaE;
    float lumaNESE = lumaNE + lumaSE;
    float lumaNWNE = lumaNW + lumaNE;

    float lumaNWSW = lumaNW + lumaSW;
    float lumaSWSE = lumaSW + lumaSE;

    bool span = abs(-2.0 * lumaW + lumaNWSW) + (abs(-2.0 * lumaM + lumaNS) * 2.0) + abs(-2.0 * lumaE + lumaNESE) >= abs(-2.0 * lumaS + lumaSWSE) + (abs(-2.0 * lumaM + lumaWE) * 2.0) + abs(-2.0 * lumaN + lumaNWNE);

    if (!span) lumaN = lumaW;
    if (!span) lumaS = lumaE;

    float lengthSign = span ? screenPixelsSize.y : screenPixelsSize.x;

    vec3 gradient = vec3(lumaN, lumaS, 1.0) - lumaM;

    bool pairN = abs(gradient.x) >= abs(gradient.y);

    float lumaNN = lumaN + lumaM;
    float lumaSS = lumaS + lumaM;

    if (!pairN) lumaNN = lumaSS;

    lengthSign = pairN ? -lengthSign : lengthSign;

    mat4x2 coords = mat4x2(0.0);

    coords[0] = texCoord;

    coords[3].x = (!span) ? 0.0 : screenPixelsSize.x;
    coords[3].y = ( span) ? 0.0 : screenPixelsSize.y;

    if (!span) coords[0].x = lengthSign * 0.5 + coords[0].x;
    if ( span) coords[0].y = lengthSign * 0.5 + coords[0].y;

    coords[1] = coords[0] - coords[3];
    coords[2] = coords[0] + coords[3];

    gradient.z = max(abs(gradient.x), abs(gradient.y)) * 0.25;

    vec2 lumaEnd = vec2(0.0);

    bvec3 done = bvec3(false, false, true);

    for (int i = 1; i <= 8; ++i) {
        
        if (!done.z) break;
    
        float inc = sqrt(i);

        if (!done.x) lumaEnd.x = luma(textureLoad(colortex0, coords[1].xy).rgb) - lumaNN * 0.5, coords[1] = -coords[3] * inc + coords[1];
        if (!done.y) lumaEnd.y = luma(textureLoad(colortex0, coords[2].xy).rgb) - lumaNN * 0.5, coords[2] =  coords[3] * inc + coords[2];

        done.x = abs(lumaEnd.x) >= gradient.z;
        done.y = abs(lumaEnd.y) >= gradient.z;

        done.z = (!done.x) || (!done.y);

    }

    bool lumaMLTZero = lumaM - (lumaNN * 0.5) < 0.0;

    bvec2 goodSpan = lessThan(lumaEnd, vec2(0.0));
    
    goodSpan.x = goodSpan.x != lumaMLTZero;
    goodSpan.y = goodSpan.y != lumaMLTZero;

    vec2 distance = vec2(span ? texCoord.x - coords[1].x : texCoord.y - coords[1].y, span ? coords[2].x - texCoord.x : coords[2].y - texCoord.y);

    float subpixel = (lumaNS + lumaWE) * 2.0 + (lumaNWSW + lumaNESE);

    subpixel = abs(subpixel * 0.125 - lumaM) * inversesqrt(max(lumaM, (lumaEnd.x + lumaEnd.y) * 0.5) + 0.5);

    subpixel = pow2((-2.0 * subpixel + 3.0) * pow2(subpixel)) * 0.75;
    
    subpixel = max((distance.x < distance.y ? goodSpan.x : goodSpan.y) ? min(distance.x, distance.y) * (-(1.0 / (distance.y + distance.x))) + 0.5 : 0.0, subpixel) * lengthSign;
    
    if (!span) texCoord.x = subpixel + texCoord.x;
    if ( span) texCoord.y = subpixel + texCoord.y;
        
}