void TAACompute(inout vec3 output0, inout vec2 newCoord) {
	
    vec4 velocity = vec4(0.0);

	velocity.xy = computeCameraVelocity(vec3(texCoord, textureLoad(depthtex1, texCoord).r));

	velocity.zw = velocity.xy;

	#ifdef MOTIONBLUR
		const float shutterRCP = 1.0 / SHUTTER_SPEED;
		const float shutterRCPHalf = shutterRCP * 0.5;

		velocity.xy = clamp01(velocity.xy + 0.5) * shutterRCP - shutterRCPHalf;

		velocity.xy /= frameTime;

		velocity.xy *= 2.0 * dither - 0.5;
	#else
		velocity.xy = vec2(0.0);
	#endif

	vec2 prevCoord = -velocity.zw + texCoord + velocity.xy;

	bool hand = textureLoad(colortex0, newCoord).a != 1;

	if (hand) velocity.xy *= MC_HAND_DEPTH >= 0.125 ? MC_HAND_DEPTH * 0.25 : 0.0;

	newCoord = newCoord + velocity.xy;

	if (textureLoad(colortex0, newCoord).a != 1 && !hand) newCoord = newCoord - velocity.xy;

	output0 = textureLoad(colortex0, newCoord).rgb;

    float weight = coordInTile(prevCoord) ? fLength(0.5 - abs(fract(prevCoord * screenPixels) - fract(texCoord * screenPixels))) : 0.0;
	
	weight = sqrt(weight / 0.75) * 0.75 + inversesqrt(weight * 9.0 + 8.0);

	mat2x3 limits = mat2x3(output0 * SHUTTER_SPEED, output0 / SHUTTER_SPEED);

	vec2 limitSize = screenPixelsSize / weight;

	for (int i = -1; i <= 1; ++i)
		for (int j = -1; j <= 1; ++j) {

			vec3 colorSample = textureLoad(colortex0, ivec2(i, j) * limitSize + newCoord).rgb;

			output0 = (output0 - colorSample) * 0.125 + output0;

			limits[0] = min(colorSample, limits[0]);
			limits[1] = max(colorSample, limits[1]);

		}
	
	#ifdef MOTIONBLUR
		limits[0] = shutterRCP * -phi + limits[0];
		limits[1] = shutterRCP *  phi + limits[1];
	#endif

	output0 = mix(output0, clamp(textureLoad(colortex3, prevCoord).rgb, limits[0], limits[1]), weight);

}