float computeFocus(in float focusPoint, in float depthInput) {
	
	depthInput = 0.1 / (1.1 - depthInput);

	return ((fStop * (focusPoint - depthInput)) / (focusPoint * (depthInput - fStop)));

}

void computeDepthOfField(inout vec3 output0, in float depthInput, in bool hand) {

	if (hand && MC_HAND_DEPTH < 0.125) return;

	float colorCorrect = 0.0;

	float distanceLimit = focalLengthMM * 0.5;

	#ifdef AUTO_FOCUS
		float focusPoint = 0.1 / (1.1 - textureLoad(colortex3, screenPixelsSize).a);
	#else
		const float focusPoint = 0.1 / (1.1 - DEPTH_VALUE);
	#endif

	vec4 focus = vec4(0.0);
	
	focus.z = computeFocus(focusPoint, depthInput);

	focus.w = clamp(smoothMask * distanceLimit + focus.z, -distanceLimit, distanceLimit);

	#ifdef BARREL_DISTORTION
		vec2 centerDist = (texCoord - 0.5) * 2.0;

		float angle = 1.0 - cos(fLength(centerDist));
		
		vec2 rotationDist = vec2(centerDist.y, -centerDist.x) * angle;

		focus.w = angle * distanceLimit * 0.5 * fSign(focus.w) + focus.w;
	#endif

	#ifdef TILT_SHIFT
		focus.w = abs(texCoord.y - 0.5) * fSign(focus.w) * distanceLimit + focus.w;
	#endif

	if (hand) focus.w = pow2(MC_HAND_DEPTH) * sqrt(MC_HAND_DEPTH) * 0.25;

	focus.xy = focus.w * vec2(0.5, aspectRatio * 0.5);

	vec3 totalSamples = vec3(0.0);

	mat3x2 sampleCoord = mat3x2(texCoord, texCoord, texCoord);

	#ifdef REFRACTION_CHROMATIC_ABERRATION
		float chromaticFocus = abs(focus.w) * 0.5;

		sampleCoord[0] = rotateVector(sampleCoord[0], -chromaticFocus);
		sampleCoord[2] = rotateVector(sampleCoord[2],  chromaticFocus);
	#endif

	focus.w = sqrt(abs(focus.w));

	#ifdef DEPTH_OF_FIELD_SAMPLE_CUTOFF
		if (!(focus.w * 8.0 - sqrt(focus.w) > 0)) return;
	#endif

	const int steps = int(sqrt(LENS_EFFECT_SAMPLES));
	const float stepRCP = RCP(steps);

	int fakeIncrement = frameCounter & (LENS_EFFECT_SAMPLES * steps - 1);

	#ifdef DEPTH_OF_FIELD_VARIABLE_SAMPLES
		float sampleCount = min(pow2(LENS_EFFECT_SAMPLES) * focus.w, LENS_EFFECT_SAMPLES) * steps;
		
		float sampleRCP = RCP(sampleCount);

		float tauScale = tau * sampleRCP;
		float piScale = internalScale.z * sampleRCP;
	#else
		const float sampleCount = LENS_EFFECT_SAMPLES * steps;
		
		const float sampleRCP = RCP(sampleCount);

		const float tauScale = tau * sampleRCP;
		const float piScale = internalScale.z * sampleRCP;
	#endif
	
	float LOD = ((sampleLOD * focus.w) * 8.0 - sqrt(focus.w)) * 2.0 * stepRCP;

	focus.xy *= phi;

	for (int i = 1; i <= RINGS; ++i) 

	for (int j = 0; j < i * sampleCount * stepRCP; ++j) {

		vec2 bokehPoints = bokehPointMap(j * steps + fakeIncrement, i, piScale, tauScale) * focus.xy;

		#ifdef BARREL_DISTORTION
			bokehPoints = rotationDist * dot(rotationDist, bokehPoints) + bokehPoints;
		#endif

		#ifdef DEPTH_AWARE_BLUR
			focus.w = computeFocus(focusPoint, textureLoad(depthtex1, texCoord + bokehPoints).r);

			if (abs(focus.z - focus.w) > abs(focus.z)) continue;
		#endif

		if (textureLoadLOD(colortex0, bokehPoints + sampleCoord[1], LOD).a > 0 && !hand) continue;
	
		totalSamples = vec3(
		textureLoadLOD(colortex0, bokehPoints * dispersion.x + sampleCoord[0], LOD).r,
		textureLoadLOD(colortex0, bokehPoints * dispersion.y + sampleCoord[1], LOD).g,
		textureLoadLOD(colortex0, bokehPoints * dispersion.z + sampleCoord[2], LOD).b) + totalSamples;

		++colorCorrect;

	}

	if (colorCorrect > 0)
	output0 = totalSamples / colorCorrect;

}