const float sampleRCP = 4.0 / LENS_EFFECT_SAMPLES;
 	
const float tauScale = tau * sampleRCP;
const float piScale = internalScale.z * sampleRCP;

vec2 bokehSize = fStop * vec2(1.0, aspectRatio) * pi;
vec2 bokehPoints = vec2(0.0);

void computeDiffraction(inout vec3 tile, in vec2 coord) {

    const float tauScale = tau / BLADES;

    bokehSize = bokehSize * sampleRCP * dither;

    float LOD = sampleLOD * pi + dither;

    for (int i = 1; i <= LENS_EFFECT_SAMPLES; ++i) {

        bokehPoints = circleMap(i * tauScale - tauScale) * (i & BLADES * LENS_EFFECT_SAMPLES - 1);

        float spikeLength = sqrt(abs((LENS_EFFECT_SAMPLES * 0.5) - fLength(bokehPoints)) / LENS_EFFECT_SAMPLES);

		if (spikeLength <= 0.0) continue;
        
        bokehPoints *= bokehSize;

        tile = linear(vec3(
        textureLoadLOD(colortex1, bokehPoints * (dispersion.x * spikeLength + 0.75) + coord, spikeLength * LOD).r,
        textureLoadLOD(colortex1, bokehPoints * (dispersion.y * spikeLength + 1.00) + coord, spikeLength * LOD).g,
        textureLoadLOD(colortex1, bokehPoints * (dispersion.z * spikeLength + 1.25) + coord, spikeLength * LOD).b)) * inversesqrt(spikeLength * LENS_EFFECT_SAMPLES) + tile;
  
    }
}

vec4 computeFlareProperties(in float distance, in vec2 coord, in vec2 offset) {
    
    distance = distance * focalLengthMM;

    vec2 ghost = 1.0 - fract(coord - offset * distance);

    return vec4(ghost, distance, coordInTile(ghost * pow8(distance)));
}

vec3 computeFlareSample(in float distance, in vec2 coord, in vec2 bokehPoints) {
    
    bokehPoints *= distance;
    
    return linear(vec3(
    textureLoadLOD(colortex1, bokehPoints * dispersion.x + coord, sampleLOD).r,
    textureLoadLOD(colortex1, bokehPoints * dispersion.y + coord, sampleLOD).g,
    textureLoadLOD(colortex1, bokehPoints * dispersion.z + coord, sampleLOD).b));
}

void computeDirtyLens(inout vec3 tile, in vec2 coord) {

    int colorCorrect = 0;

    const float upscaledRCP = sampleRCP * 0.75;
    const float upscaledPiScale = piScale * 0.75;
    const float upscaledTauScale = tauScale * 0.75;
    const float LOD = sampleLOD * 2.0;

    for (int i = 1; i <= RINGS; ++i)

    for (int j = 0; j < i * LENS_EFFECT_SAMPLES * 0.5; ++j) {

        bokehPoints = bokehPointMap(dither * upscaledRCP + j, i, upscaledPiScale, upscaledTauScale) * bokehSize;

        tile = linear(vec3(
        textureLoadLOD(colortex2, bokehPoints * dispersion.x + coord, LOD).b, 
        textureLoadLOD(colortex2, bokehPoints * dispersion.y + coord, LOD).b, 
        textureLoadLOD(colortex2, bokehPoints * dispersion.z + coord, LOD).b)) + tile;

        ++colorCorrect;
    }

    tile /= colorCorrect;
}