void computeWaterRefraction(inout vec3 output0, in mat2x3 waves, in positionStruct Position) {

    vec2 waterCoord = texCoord;

    #ifdef WATER_REPROJECTION
        vec3 clipPosition = refract(Position.viewSpacePosition[2], waves[1], 0.75) * 1.333 + Position.viewSpacePosition[0];
        
        vec2 projectedCoord = (diagonal2(gbufferProjection) * clipPosition.xy + gbufferProjection[2].xy) / -clipPosition.z * 0.5 + 0.5;

        waterCoord = mix(projectedCoord, texCoord, smoothstep(0.5, 1.0, 1.0 - projectedCoord) + smoothstep(0.5, 1.0, projectedCoord));
    #endif

    vec2 screenWave = waves[0].xy * waves[0].z;

    int IDs = decodeIDs(textureLoad(colortex2, screenWave * 0.150 + waterCoord).b).r;

    if (water) {

        float LOD = fLength(screenWave) * 32.0;

        output0.r = textureLoadLOD(colortex1, screenWave * 0.150 + waterCoord, LOD).r;
        output0.g = textureLoadLOD(colortex1, screenWave * 0.125 + waterCoord, LOD).g;
        output0.b = textureLoadLOD(colortex1, screenWave * 0.100 + waterCoord, LOD).b;

        output0 = linear((output0 * rcp16bit) * 0.5 + 0.5);

    }

}

void computeUnderwaterRefraction(inout vec3 output0, inout vec3 output1) {

    vec2 screenWave = generate2DNoise(texCoord * 32.0 + time) * vec2(0.25, -0.25);

    screenWave = screenWave * screenWave;

    vec2 boundaryCoord = screenWave * 0.1 + texCoord;

	if (coordInTile(boundaryCoord)) {

        float LOD = fLength(screenWave) * 32.0;

        output0.r = textureLoadLOD(colortex1, screenWave * 0.150 + texCoord, LOD).r;
        output0.g = textureLoadLOD(colortex1, screenWave * 0.125 + texCoord, LOD).g;
        output0.b = textureLoadLOD(colortex1, screenWave * 0.100 + texCoord, LOD).b;

        output0 = linear((output0 * rcp16bit) * 0.5 + 0.5);

        output1 = textureLoad(colortex2, screenWave * 0.150 + texCoord).rgb;

    }
}