void computeRefraction(inout mat3x2 newCoord, inout float LOD) {

    float rainWave = 0.0;

    vec3 heatWave = vec3(0.0);

    #ifdef REFRACTION_RAIN_DROPS
        if (rainStrength > 0.0) {

            const mat3x2 weights = mat3x2(vec2(0.25, 0.18), vec2(0.16, 0.14), vec2(0.125, 0.11));

            mat4x2 drops = mat4x2(0.0);

            drops[3] = vec2(5.0, 20.0);

            vec2 rainCoord = texCoord;
            
            rainCoord = generate2DNoise(vec2(texCoord.x, texCoord.y * 32)) * 0.01625 + rainCoord;

            vec2 waveBase = vec2(rainCoord.x * drops[3].x + rainCoord.x, rainCoord.y) * 0.5 + vec2(0.0, time * 0.25);

            rainWave = generate2DNoise(waveBase * 16.0) * 1.2 + rainWave;

            for (int i = 0; i < 3; ++i) {

                drops[3].x += i;

                drops[0] = weights[i];

                drops[1] = vec2(rainCoord.x * drops[3].x + rainCoord.x, rainCoord.y) * drops[0].x;

                drops[2] = vec2(0.0, -time * drops[0].y);

                drops[3].y = drops[3].y * i + 4.0;

                rainWave = generate2DNoise(drops[1] * drops[3].y + drops[2]) * rainWave - 0.1;

            }

            rainWave = clamp01(rainWave * clamp01(rainStrength * eyeBrightness.y - 220.0)) * 0.5;

            rainWave /= rainWave * 16.0 + 1.0;

        }
        
        if (!coordInTile(rainWave * 0.25 + texCoord)) rainWave *= 0;
    #endif

    #ifdef REFRACTION_HEAT_WAVES
        LOD = computeMaxLOD(log2(screenPixels));

        vec2 correctCoord = (LOD * 16) * screenPixelsSize + texCoord;

        heatWave.z = luma(textureLoadLOD(colortex1, correctCoord, LOD).rgb) * textureLoadLOD(colortex2, correctCoord, LOD).r;
       
        float waveTime = time * 3.0;

        heatWave.x = sin(texCoord.x * 250.0 + waveTime);
        heatWave.y = cos(texCoord.y * 100.0 + waveTime);

        heatWave *= heatWave.z * inversesqrt(heatWave.z * 4096.0 + 8.0);
    #endif

    LOD = clamp01(floor(rainWave * 32.0) * 0.25) * 4.0;

    vec2 refraction = heatWave.xy * vec2(0.5, 1.5) + rainWave;
    
    #ifdef REFRACTION_LENS_DISTORTION
        newCoord[1] = texCoord * 2.0 - 1.0;

        float centerDistance = fLength(newCoord[1] * (1.0 - focalLengthMM * focalLengthMM));
        
        float theta = atan(newCoord[1].y, newCoord[1].x);

        newCoord[1] = 0.5 * (centerDistance * vec2(cos(theta), sin(theta)) + 1.0);
    
        newCoord[1] = centerDistance > 1.0 ? texCoord : mix(newCoord[1], texCoord, centerDistance);
    #else
        newCoord[1] = texCoord;
    #endif

    #ifdef REFRACTION_CHROMATIC_ABERRATION
        float pixelLength = fLength(screenPixelsSize) * aspectRatio * fLength(abs(newCoord[1] * 1.0 - 0.5));

        newCoord[0] = rotateVector(newCoord[1],  pixelLength);
        newCoord[2] = rotateVector(newCoord[1], -pixelLength);
    #else
        newCoord[0] = newCoord[1];
        newCoord[2] = newCoord[1];
    #endif
    
    newCoord[0] = refraction * 0.125 + newCoord[0];
    newCoord[1] = refraction * 0.100 + newCoord[1];
    newCoord[2] = refraction * 0.075 + newCoord[2];

}