float cloudFBM2D(in vec2 cloudPlane, in float cloudMovement, out mat4x2 detailCoords) {

	detailCoords[0] = cloudPlane * 0.014;

	detailCoords[0] += generate2DNoise(detailCoords[0]) * 0.3 - 0.15;

	detailCoords[0].x = detailCoords[0].x * 0.25 + cloudMovement;
	
	float cloud = -generate2DNoise(detailCoords[0]);
	
	detailCoords[1] = detailCoords[0] * 2.0 - cloud * vec2(0.5, 1.35) + vec2(cloudMovement, 0.0);
	
	cloud = generate2DNoise(detailCoords[1]) * noiseWeights.x + cloud;
	
	detailCoords[2] = detailCoords[1] * vec2(12.0, 2.65) + cloudMovement * vec2(3.0, 0.55) - cloud * vec2(1.5, 0.75);
	
	cloud = generate2DNoise(detailCoords[2]) * noiseWeights.y + cloud;
	
	detailCoords[3] = detailCoords[2] * 3.0 + cloudMovement;
	
	cloud = generate2DNoise(detailCoords[3]) * noiseWeights.z + cloud;
	
	cloud = generate2DNoise(detailCoords[3] * 3.0 + cloudMovement) * 0.022 + cloud;

	return noiseWeights.w - cloud;
	
}

void computeTwoDimensionalClouds(inout vec3 output0, in vec3 worldSpacePosition, in vec3 rayleigh, in vec2 horizon, in vec2 lightAura, in float cloudMovement, inout float cloudSurface) {

	const float cloudHeight = 384.0;

    if (cloudHeight <= cameraPosition.y && worldSpacePosition.y >= 0.0 || worldSpacePosition.y <= 0.0) return;

	vec2 cloudPlane = worldSpacePosition.xz / worldSpacePosition.y * (cloudHeight - cameraPosition.y) + cameraPosition.xz;

	vec4 cloudOutput = vec4(0.0);
	mat4x2 detailCoords = mat4x2(0.0);

	cloudOutput.a = coverageCurve(cloudFBM2D(cloudPlane, cloudMovement, detailCoords) * cloudCoverage, cloudCoverage);

	vec2 lightOffset = vec2(dot(lightPosition[1], gbufferModelView[0].xyz), dot(lightPosition[1], gbufferModelView[2].xyz)) * 0.2;

	float detail = -generate2DNoise(detailCoords[0] + lightOffset);
	
	detail = generate2DNoise(detailCoords[1] + lightOffset) * noiseWeights.x + detail;
	detail = generate2DNoise(detailCoords[2] + lightOffset) * noiseWeights.y + detail;
	detail = generate2DNoise(detailCoords[3] + lightOffset) * noiseWeights.z + detail;

	detail = pow6((inverseCoverage + 1.2) - coverageCurve(noiseWeights.w - detail, cloudCoverage)) * cloudLight * pow2(cloudOutput.a * 2.0);

	cloudOutput.rgb = mix(rayleigh, rayleigh * cloudLight, detail);
	
	output0 = mix(output0, linear(cloudOutput.rgb), cloudOutput.a * horizon.y * (cloudLight * sqrt(cloudCoverage) + inverseCoverage) * (1.0 - cloudSurface));
	
	cloudSurface = clamp01(cloudOutput.a * cloudOutput.a + cloudSurface);

	
}