float cloudFBM3D(in vec3 worldPosition, in float cloudMovement) {

    float cloud = 0.0;

    worldPosition = worldPosition * 0.012;

    worldPosition.x = worldPosition.x * 0.6 + cloudMovement;

    cloud = -generate3DNoise(worldPosition);

    worldPosition = worldPosition *  vec3(1.0, 2.0, 2.0) + cloudMovement;

    cloud = generate3DNoise(cloud * -vec3(1.0, 2.0, 2.0) + worldPosition) * noiseWeights.x + cloud;

    worldPosition = worldPosition *  vec3(6.0, 4.0, 4.0) + cloudMovement;

    cloud = generate3DNoise(cloud * -vec3(6.0, 4.0, 4.0) + worldPosition) * noiseWeights.y + cloud;

    worldPosition = worldPosition *  vec3(6.0, 4.0, 4.0) + cloudMovement;

    cloud = generate3DNoise(cloud * -vec3(6.0, 4.0, 4.0) + worldPosition) * noiseWeights.z + cloud;

    return noiseWeights.w - cloud;

}

void computeThreeDimensionalClouds(inout vec3 output0, in vec3 worldSpacePosition, in float yRCP, in vec3 rayleigh, in vec2 horizon, in vec2 lightAura, in float cloudMovement, inout float cloudSurface) {
    
    const float cloudLowerHeight = 256.0;
    const float cloudUpperHeight = 304.0;

    if (cloudLowerHeight <= cameraPosition.y && worldSpacePosition.y >= 0.0 || worldSpacePosition.y <= 0.0) return;

    const float cloudDepth = cloudUpperHeight - cloudLowerHeight;

    mat4x3 cloudPlanes = mat4x3(0.0);

    cloudPlanes[0] = worldSpacePosition * ((cloudLowerHeight - cameraPosition.y) * yRCP);
    cloudPlanes[1] = worldSpacePosition * ((cloudUpperHeight - cameraPosition.y) * yRCP);    
    cloudPlanes[2] = (cloudPlanes[1] - cloudPlanes[0]) * 0.125;
    cloudPlanes[3] = cloudPlanes[2] * dither + (cloudPlanes[0] + cameraPosition);

    const float cloudCenter = cloudDepth * 0.5 + cloudLowerHeight;
    const float cloudDepthHalf = RCP(cloudDepth * 0.5);
    const float cloudFalloff = RCP(cloudDepth - cloudDepth * 0.25);

    float weight = 0.0;
    float FBM = 0.0;

    vec4 cloudOutput = vec4(0.0);
    
    for (float i = 0.0; i < 1.0 && cloudOutput.a < 1.0; i += 0.125, cloudPlanes[3] += cloudPlanes[2]) {

        weight = (inverseCoverage + 1.0) - abs(cloudPlanes[3].y - cloudCenter) * cloudDepthHalf;
        
        FBM = coverageCurve(cloudFBM3D(cloudPlanes[3], cloudMovement) * weight, cloudCoverage);

        FBM /= FBM * 0.125 + 0.25;
    
        weight = pow6(abs(cloudPlanes[3].y - cloudLowerHeight) * cloudFalloff * sqrt(FBM));

        cloudOutput.rgb = mix(rayleigh, rayleigh * cloudLight, weight) * (2.0 * clamp01(0.5 - cloudOutput.a) * FBM) + cloudOutput.rgb;

        cloudOutput.a = FBM * 0.125 + cloudOutput.a;

    }
    
    output0 = mix(output0, linear(cloudOutput.rgb * 0.125), sqrt(cloudOutput.a * horizon.y * (cloudLight * sqrt(cloudCoverage) + inverseCoverage) * (1.0 - cloudSurface) * 3.0) * 0.333);

    cloudSurface = clamp01(cloudOutput.a * cloudOutput.a + cloudSurface);

}