void computeThreeDimensionalRays(inout float CR, in vec2 compositeCoord, in positionStruct Position) {

    Position.worldSpacePosition = fNormalize(Position.worldSpacePosition);

    Position.shadowSpacePosition = transMAD(shadowProjMatrix, Position.worldSpacePosition + gbufferModelViewInverse[3].xyz);
    
    vec3 shadowStep = diagonal3(shadowProjection) * (mat3(shadowModelView) * Position.worldSpacePosition);

    float playerDistance = fLength(Position.viewSpacePosition);

    const float rayIncrement = 64.0 / CREPUSCULAR_RAYS_SAMPLES;

    for (float i = dither * rayIncrement + RCP(rayIncrement); i < CREPUSCULAR_RAYS_3D_DEPTH && playerDistance > i; i += rayIncrement) {

        vec3 samplePosition = distortShadowSpace(shadowStep * i + Position.shadowSpacePosition, shadowDistanceSize, inverseDistanceSize) * 0.5 + 0.5;

        CR += shadowLoad(shadowtex1, samplePosition).r;

        #if defined SHADOW_WATER_CAUSTICS && defined CREPUSCULAR_RAYS_CAUSTIC_RAYS
            if (abs(textureLoad(shadowcolor0, samplePosition.xy).a - 0.25) < rcp8bit && (samplePosition.z - textureLoad(shadowtex0, samplePosition.xy).r) > shadowPixelsSize)
            CR = textureLoad(shadowcolor0, samplePosition.xy).r * CR + CR;
        #endif

    }

    CR = RCP(CREPUSCULAR_RAYS_3D_DEPTH * 2.0) * rayIncrement * CR;
}

void computeTwoDimensionalRays(inout float CR, in vec2 compositeCoord, in positionStruct Position) {

    const float sampleRCP = RCP(CREPUSCULAR_RAYS_SAMPLES);

	vec3 projectedLightPosition = projMAD3(gbufferProjection, lightPosition[1]);

    vec2 deltaPosition = (compositeCoord - ((projectedLightPosition.xy / projectedLightPosition.z) * 0.5 + 0.5)) * sampleRCP;

    float CR2D = 0.0;

	for (int i = 0; i < CREPUSCULAR_RAYS_SAMPLES; ++i)
    CR2D = step(textureLoad(colortex2, deltaPosition * dither + (compositeCoord - deltaPosition * i)).b, 0.0) * sampleRCP + CR2D;

    CR = mix(CR, CR2D * 0.5, clamp01(dot(lightPosition[1], fNormalize(Position.viewSpacePosition))));
} 