#define atmosphereHeight 8228.0
#define earthRadius 6371000.0

#define mieDistribution 1.8
#define mieCoefficient 1.78e-5

#define ozoneCoefficient (vec3(1.426, 7.298, 0.356) * 6.3e-7)
#define ozoneMultiplier 1.86
#define ozoneHeight 30000.0

// ( Riley, Ebert, Kraus )
//#define rayleighCoefficient vec3(5.8e-6  , 1.35e-5 , 3.31e-5 )
// ( Bucholtz )
//#define rayleighCoefficient vec3(4.847e-6, 1.149e-5, 2.87e-5 ) 
// ( Thalman, Zarzana, Tolbert, Volkamer )
//#define rayleighCoefficient vec3(5.358e-6, 1.253e-5, 3.062e-5)
// ( Penndorf )
//#define rayleighCoefficient vec3(5.178e-6, 1.226e-5, 3.06e-5 )
// ( Jodie )
//#define rayleighCoefficient vec3(4.593e-6, 1.097e-5, 2.716e-5)
// ( Vignette )
#define rayleighCoefficient vec3(4.395e-6, 1.083e-5, 3.267e-5)

#define rayleighDistribution 8.0

#define phaseRayleigh(a) (0.4 * a + 1.12)
#define phaseMie(x, g) (pow2(g) * -0.25 + 0.25) * exp2(-(log2(-2.0 * (g * x) + (g * g + 1.0)) * rcpln2))

const mat3 skyCoeffs = mat3(rayleighCoefficient * rcpln2, 1.12 * vec3(mieCoefficient) * rcpln2, ozoneCoefficient * ozoneMultiplier * rcpln2);
const vec4 sr = earthRadius + vec4(atmosphereHeight, atmosphereHeight * mieDistribution / rayleighDistribution, ozoneHeight, ozoneHeight + atmosphereHeight);

vec3 skyThickness(in vec3 rd) {
 
    float b = dot(rd, -upDirection) * earthRadius;

    vec4 z = b + sqrt(sr * sr + (b * b - pow2(earthRadius)));

    return vec3(z.xy, z.w - z.z);
}

vec3 sunCoeff = skyCoeffs * skyThickness(lightPosition[0]);
vec3 sunAbsorb = exp2(-sunCoeff);

vec3 computeSky(in vec3 V) {

    vec3 thickness = skyThickness(V);
	
    float VdotL = dot(V, lightPosition[0]);

    vec3 viewCoeff = skyCoeffs * thickness;

    vec4 scatter = vec4(0.0);

    scatter.xyz = thickness.x * phaseRayleigh(VdotL) * rayleighCoefficient;
    scatter.w = thickness.y * phaseMie(VdotL, thickness.y) * mieCoefficient;

    vec3 absorption = abs(sunAbsorb - exp2(-viewCoeff)) / (abs(sunCoeff - viewCoeff) * ln2 + rcp32bit);

    vec3 sky = vec3(1.0, 0.899, 0.828) * (scatter.xyz + scatter.w) * absorption;
    
    return mix(sky, luma(sky * 0.25) * absorption, rainStrength * 0.95);
}