#version 120

/* DRAWBUFFERS:0 */

#define PIXEL

varying vec4 biomeColor;

varying vec3 vertexPos;

varying vec3 vertexNormal;

flat varying float vertexID;

uniform sampler2D texture;

uniform float frameTimeCounter;

#include "/lib/utilities.glsl"

#include "/lib/utility/water/waterWaves.glsl"

void main() {

    vec4 color = texture2D(texture, texCoord) * biomeColor;

    float IDs = round(vertexID);

    if (glass) color.a = floor(color.a + 0.5) - 0.5;
    
    if (colored) color.a *= 0.5;

    if (waterSource) {
        #if defined SHADOW_WATER_CAUSTICS && defined SHADOW_COLORED_SHADOWS
            mat2x3 waves = mat2x3(0.0);
            
            waves[0].y = getWaves(vertexPos);
            waves[0].x = getWaves(vertexPos + vec3(wavesDifferentialCoeff, 0.0, 0.0));
            waves[0].z = getWaves(vertexPos + vec3(0.0, 0.0, wavesDifferentialCoeff));

            waves[1].xy = (waves[0].y - waves[0].xz) / wavesDifferentialCoeff;

            waves[1].z = sqrt(abs(1.0 - dotX(waves[1].xy)));

            waves[1].z = dot(waves[1].xy, waves[1].zz);

            waves[1] = fNormalize(waves[1]);

            vec3 normalizedPos = fNormalize(vertexPos);

            vec3 refractPos = refract(normalizedPos, waves[1], 0.75);
            
            float caustic = fPow(fLength(dFdx(normalizedPos) * dFdy(normalizedPos)) * fInverseLength(dFdx(refractPos) * dFdy(refractPos)), 0.125);

            color.rgb = caustic * exp2(-waterColor.rgb) / 0.75;
        #else
            color.rgb = exp2(-waterColor.rgb);
        #endif
            
        color.a = 0.25;

    }

    gl_FragData[0] = color;
    gl_FragData[1] = vec4(vertexNormal * 0.5 + 0.5, 1.0);
}